#################################################################################
# Created       : Martin Foster
# Modified      : 09-Sep-2003
#################################################################################
#
# Options - Handling sub-routines for use of the pseudo option arrays   
# Copyright (C) 2000-2003  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 204 - 817, 5th Street NE
#                 Calgary, Alberta
#                 T2E 3W9
#
#################################################################################

package eRealms::Option;				# Package Name

#################################################################################
# Sub-Routines
#################################################################################

#####################
# Constructor
#
# Allows to class to be exported and thus spawn objects as a result

sub new
{
	my $class = shift;
	my $self  = { };

	# Bless $self into designated class
	bless $self, $class;

	# Return self
	return $self;
}


#####################
# Find
#
# Takes two scalars in delimited form.  And will find the corresponding value to
# the search value.  If none is found then the first element of the array is 
# sent back to avoid undefs.

sub Find							# Master, Slave Value
{
	####################
	# Data members

	my $self     = shift;					# Reference to self
	my $master   = shift;					# List of master
	my $slave    = shift;					# List of slave
	my $value    = shift;					# Value to seach for

	my @master; 						# List of searching
	my @slave;						# Associated filenames


	#####################
	# Program area

	# Retreive colours and hex
	$self->Split(\@master, $master);
	$self->Split(\@slave, $slave);

	# Find lenght of colors
	$_ = @master;

	# Determine proper index
	for (my $count; $count < $_; $count++)
	{
		# And return once found
		if ($master[$count] eq $value)
		{
			return $slave[$count];
		}
	}

	# Retrun default otherwise
	return $slave[0];
}

# Find Javascript
sub FindJavascript
{ return Find(shift, shift, 'yes:no:default', shift); } 

# Find Chat Preferences
sub FindChatPrefs
{ return Find(shift, shift, 'basic:framed:stream:autoscroll:default', shift); } 

# Find Clone
sub FindClone
{ return Find(shift, shift, 'tag:title:both', shift); } 

# Find Formatting Options
sub FindFormatting
{ return Find(shift, shift, 'complete:imageless:spartan:default', shift); } 

# Find Ignore Scope
sub FindScope
{ return Find(shift, shift, 'yes:no:yes', shift); }

# Privacy Options
sub FindPrivacy
{ return Find(shift, shift, 'yes:no:global:default', shift); } 

# Access control lists
sub FindACL
{ return Find(shift, shift, '0:1:2:4:8', shift); }

# Ignore lists
sub FindIgnore
{ return Find(shift, shift, 'single:global:visual:behead:optics', shift); }


# Gallery Specific
# Find Comment Display
sub GalleryComment
{ return Find(shift, shift, 'comment:rating:combo:none', shift); }

# Find Comment Display Reversed
sub GalleryCommentRev
{ return Find(shift, 'comment:rating:combo:none', shift, shift); }

# Find search field
sub GalleryField
{ return Find(shift, shift, 'wrk.WorkDescription:wrk.WorkName:gal.GalleryPenName', shift); }

# Find Sorting Method
sub GallerySort
{ return Find(shift, shift, 'alpha:date:rating', shift); }

# Find Sorting Method Reversed
sub GallerySortRev
{ return Find(shift, 'alpha:date:rating', shift, shift); }

# Find Gallery Type
sub GalleryType
{ return Find(shift, shift, 'image:text', shift); }

# Find Gallery Type Reversed
sub GalleryTypeRev
{ return Find(shift, 'image:text', shift, shift); }


# Registration Specific
# Find Javascript
sub RegJavascript
{ return Find(shift, shift, 'yes:no:yes', shift); } 

# Find Chat Preferences
sub RegChatPrefs
{ return Find(shift, shift, 'basic:framed:stream:autoscroll:stream', shift); } 

# Find Formatting Options
sub RegFormatting
{ return Find(shift, shift, 'complete:imageless:spartan:complete', shift); } 


#####################
# OptionSplit
#
# Splits options list into seperate elements of array.   This is primarily to hide
# the split statement, or be able to change it if needs arise.

sub Split
{
	#####################
	# Data members

	my $self     = shift;					# Reference to self
	my $array    = shift;					# Reference to array
	my $data     = shift; 					# Retreived line


	#####################
	# Program area
	@{ $array } = split(/:/, $data);			# Split line
}

1;								# Return true
